<?php

/*
 * @software   IniParser
 * @version    0.2.2-rc
 * @author     James Brumond
 *
 * Copyright 2010 James Brumond
 * Dual licensed under MIT and GPL
 */

if (__FILE__ == $_SERVER['SCRIPT_FILENAME']) die("Bad Load Order");



/*
 * @const  INIPARSER_DEFAULT_COMMENT
 */
	define("INIPARSER_DEFAULT_COMMENT", ";");

/*
 * @const  INIPARSER_DEFAULT_DELIMITER
 */
	define("INIPARSER_DEFAULT_DELIMITER", "=");

/*
 * @const  INIPARSER_DEFAULT_ARRAYS
 */
	define("INIPARSER_DEFAULT_ARRAYS", 1);

/*
 * @const  INIPARSER_DEFAULT_HASHES
 */
	define("INIPARSER_DEFAULT_HASHES", 1);

/*
 * @const  INIPARSER_DEFAULT_INHERITENCE
 */
	define("INIPARSER_DEFAULT_INHERITENCE", 1);

/*
 * @const  INIPARSER_OPTION_ARRAYS
 */
	define("INIPARSER_OPTION_ARRAYS", 10);

/*
 * @const  INIPARSER_OPTION_HASHES
 */
	define("INIPARSER_OPTION_HASHES", 20);

/*
 * @const  INIPARSER_OPTION_INHERITENCE
 */
	define("INIPARSER_OPTION_INHERITENCE", 30);

/*
 * @const  INIPARSER_COMMENT
 */
	define("INIPARSER_COMMENT", '/%%C%%.*/');

/*
 * @const  INIPARSER_KEY_VALUE
 */
	define("INIPARSER_KEY_VALUE", '/\s*([a-zA-Z0-9_]+)\s*%%D%%\s*([\'"]?)([a-zA-Z0-9_\. ]+)([\'"]?)\s*/');

/*
 * @const  INIPARSER_ARRAY
 */
	define("INIPARSER_ARRAY", '/([a-zA-Z0-9_]+)\s*%%D%%\s*\((.*?)\)/');

/*
 * @const  INIPARSER_HASH
 */
	define("INIPARSER_HASH", '/([a-zA-Z0-9_]+)\s*%%D%%\s*\{(.*?)\}/s');

/*
 * @const  INIPARSER_SECTION
 */
	define("INIPARSER_SECTION", '/\[\s*(([a-zA-Z0-9_]+)(\s*:)?(\s*[a-zA-Z0-9_]+)?)\s*\]/');



/*
 * @include  class.exceptions.php
 */
	require_once "class.exceptions.php";



/*
 * @class IniFile
 */
class IniFile {

/*
 * Statics
 */
	
	protected static $except = null;

/*
 * Private Properties
 */
	
	protected $file_path         = null;
	protected $contents          = null;
	protected $storage           = null;
	protected $exceptions        = null;
	protected $comment           = INIPARSER_DEFAULT_COMMENT;
	protected $delimiter         = INIPARSER_DEFAULT_DELIMITER;
	protected $arrays            = INIPARSER_DEFAULT_ARRAYS;
	protected $hashes            = INIPARSER_DEFAULT_HASHES;
	protected $parse_inheritence = INIPARSER_DEFAULT_INHERITENCE;

/*
 * Private Methods
 */

	// check if $this->file_path is a valid text file
	protected function is_valid_file() {
		if (is_string($this->file_path))
			return (file_exists($this->file_path) && is_file($this->file_path));
		return false;
	}
	
	// read the file to the contents variable
	protected function read_file() {
		if ($this->is_valid_file()) {
			return file_get_contents($this->file_path);
		}
		$this->exceptions->throw_exception("cannot read invalid file");
		return false;
	}
	
	// split a conf file into sections
	protected function section($str) {
		if (is_string($str)) {
			$sections = array();
			$data = preg_split(INIPARSER_SECTION, $str);
			$sections['_global'] = array_shift($data);
			while (preg_match(INIPARSER_SECTION, $str, $match)) {
				$sections[$match[1]] = array_shift($data);
				$str = preg_replace(INIPARSER_SECTION, "", $str, 1);
			}
			
			return $sections;
		}
	}
	
	// handles inheriting values from parent sections
	protected function parse_section_inheritence($data) {
		$new_data = array();
		foreach ($data as $section => $section_data) {
			$sects = preg_split('/\s*:\s*/', $section);
			if (count($sects) == 2 && array_key_exists($sects[1], $data)) {
				$tmp = $data[$sects[1]];
				foreach ($section_data as $key => $value)
					$tmp[$key] = $value;
				$new_data[$sects[0]] = $tmp;
			} else {
				$new_data[$section] = $section_data;
			}
		}
		return $new_data;
	}
	
	// parse and return $this->contents
	protected function parse($str = null) {
		if ($str === null) {
			if ($this->contents || $this->read_file()) {
				$contents = $this->contents;
			} else {
				$this->exceptions->throw_exception("cannot parse invalid file");
				return false;
			}
		} else {
			$contents = $str;
		}
		// get patterns
		$comment = str_replace("%%C%%", $this->comment, INIPARSER_COMMENT);
		$array = str_replace("%%D%%", $this->delimiter, INIPARSER_ARRAY);
		$hash = str_replace("%%D%%", $this->delimiter, INIPARSER_HASH);
		$keyvalue = str_replace("%%D%%", $this->delimiter, INIPARSER_KEY_VALUE);
		// remove comments
		$contents = preg_replace($comment, "", $contents);
		// remove blank lines
		$contents = explode("\n", $contents);
		$lines = array();
		foreach ($contents as $line) {
			$line = trim($line);
			if (! empty($line)) $lines[] = $line;
		}
		$contents = implode("\n", $lines);
		$data = array();
		// seperate into sections
		$sections = $this->section($contents);
		foreach ($sections as $section => $section_contents) {
			// parse for arrays
			if ($this->arrays) {
				while (preg_match($array, $section_contents, $match)) {
					$data[$section][$match[1]] = preg_split('/\s*,\s*/', trim($match[2]));
					$section_contents = preg_replace($array, "", $section_contents, 1);
				}
			}
			// parse for hashes
			if ($this->hashes) {
				while (preg_match($hash, $section_contents, $match)) {
					$data[$section][$match[1]] = $this->parse($match[2]);
					$section_contents = preg_replace($hash, "", $section_contents, 1);
				}
			}
			// parse for normal key/value pairs
			while (preg_match($keyvalue, $section_contents, $match)) {
				$data[$section][$match[1]] = $match[3];
				$section_contents = preg_replace($keyvalue, "", $section_contents, 1);
			}
		}
		// deal with sections/subsections
		if ($this->parse_inheritence)
			$data = $this->parse_section_inheritence($data);
		$finished_result = $data['_global'];
		foreach ($data as $section => $section_data) {
			if ($section != '_global')
				$finished_result[$section] = $section_data;
		}
		return (object) $finished_result;
	}

/*
 * Magic Methods
 */
	
	public function __construct($file_path = null) {
		// get the exception handler
		if (! self::$except)
			self::$except = new Except;
		$this->exceptions =& self::$except;
		
		// see if we have a file path
		if (is_string($file_path))
			$this->file_path = $file_path;
			
		// check if we can parse the file
		if ($this->is_valid_file()) {
			$this->contents = $this->read_file();
			$this->storage = $this->parse();
		}
	}
	
	public function __get($name) {
		if ($name == 'read') {
			return $this->storage;
		} else {
			return null;
		}
	}

/*
 * Public Properties
 */
	
	//...

/*
 * Public Methods
 */
	
	public function delimiter($str) {
		if (is_string($str)) {
			$this->delimiter = $str;
			return true;
		}
		return false;
	}
	
	public function comment($str) {
		if (is_string($str)) {
			$this->comment = $str;
			return true;
		}
		return false;
	}
	
	public function set_option($option, $flag) {
		switch ($option) {
			case INIPARSER_OPTION_ARRAYS:
				$this->arrays = !!$flag;
			break;
			case INIPARSER_OPTION_HASHES:
				$this->hashes = !!$flag;
			break;
			case INIPARSER_OPTION_INHERITENCE:
				$this->parse_inheritence = !!$flag;
			break;
			default:
				$this->exceptions->throw_exception("invalid option");
			break;
		}
	}

	public function refresh() {
		$this->contents = $this->read_file();
		$this->storage = $this->parse();
	}

	public function set_path($path) {
		if (is_string($file_path)) {
			$this->file_path = $file_path;
			return $this->refresh();
		} else {
			return false;
		}
	}

}

?>
